require(grDevices)

downsample <- function(x, rres,cres,
  ridx = 1:nrow(x), cidx = 1:ncol(x), rcenter=TRUE, ccenter=TRUE)
{
  m <- length(ridx); n <- length(cidx);
  my <- (m+rres-1) %/% rres; ny <- (n+cres-1) %/% cres; 
  r <- .C("R_downsample",
    as.integer(nrow(x)),as.integer(ncol(x)),as.double(x),
    as.integer(rres),as.integer(cres),
    as.integer(m), as.integer(n),
    as.integer(ridx-1), as.integer(cidx-1),
    as.integer( rcenter + 2*ccenter ),
    y = double(my*ny), meany = double(1), sdy = double(1),
    NAOK=TRUE,DUP=FALSE);
  dim(r$y)=c(my,ny);
  attr(r$y,"mean") <- r$meany
  attr(r$y,"sd") <- r$sdy
  r$y
}

labelpack <- function(x,space = NULL, spacefrac = 0.05,
  minx = min(x), maxx = max(x), maxiter = 50){
  n <- length(x)
  o <- order(x)

  if(missing(space))
    space <- spacefrac*(maxx-minx)
    
  r <- .C("R_labelpack",as.integer(n),as.double(x[o]),
    as.double(space),as.double(minx),as.double(maxx),
    as.integer(maxiter),y = double(n))
  r$y[o] <- r$y
  r$y }

# red-black-green gradient
rg.colors <- c(rgb(0,seq(1,.1,-.1),0), rgb(0,0,0), rgb(seq(.1,1,.1),0,0) ) 

# red-white-blue gradient
rb.colors <- c(hsv(0.66,seq(1,.1,-.1),1), rgb(1,1,1), hsv(0,seq(.1,1,.1),1) ) 

# black-and-white
bw.colors <- gray(seq(21,0,-1)/21)

coldmap <- function(..., clust=NULL,
  dlab=NULL,
  clab = NULL,
  col.clab=NULL,
  cex.clab=.8,
  rannot = NULL,
  col=rb.colors,
  saturation=.666,
  downsampling = NULL,
  wa=NULL, wb=NULL,
  chang = 0.0,
  rhang = 0.05,
  gutter=NULL, proportional = TRUE,
  rpwcorr=TRUE,
  cpwcorr=FALSE,
  rdw = 3, cdw =3,
  rannot.width = 1.5,
  rgap = 0.5,
  rticks = NULL, cticks = NULL,
  corder = NULL
  )
{
  d <- list(...)
  K <- length(d)
  if(is.null(clust))
    clust <- nclust2(...,rpwcorr=rpwcorr,cpwcorr=cpwcorr)


  par0 <- par(no.readonly=TRUE)
  on.exit(par(par0))

  # make per dataset ordering
  nrow.d <- sapply(d,function(u)nrow(u))
  b.row <- cumsum(nrow.d)
  a.row <- c(0,b.row[-K])+1
  N <- sum(nrow.d)


  if( K > 1 )
    {
    if(is.null(gutter)) gutter <- 0.05 * mean(nrow.d) 
    if( proportional )
      w <- nrow.d + gutter
    else
      w <- rep(mean(nrow.d),K) + gutter
    }
  else
    {
    w <- 1
    if( is.null(gutter)) gutter <- 0
    }
  if( proportional == FALSE )
    gutter <- 0
  else
    rgap <- 0

  L <- array(
    c(0,seq(1,2*K,2),0,2*K+2,seq(2,2*K,2),2*K+1,rep(0,K+2)),
    dim=c(K+2,3))

  # for rannot
  for(k in 1:K)
    L[1+k,3] = 2*K+2+k
  L[1,3] <- 2*K+2+K+1

  layout(
    L,
    widths=c(lcm(rdw),10,lcm(rannot.width)), heights=c(lcm(2),w,lcm(cdw))  )

  # draw individual heatmap

  cc <- clust$cclust
  if(is.null(corder))
    nu <- nrow(cc$unicol)
  else
    {
    if( !is.character(corder) )
      stop("corder has to be vector of characters.")
    nu <- length(corder)
    }

  if( is.null(wa) || wa < 1 )
    wa <- 1
  if( is.null(wb) || wb > nu )
    wb <- nu

  if( is.null(downsampling[2]) )
    cres <- (wb-wa+1) %/% 400 + 1
  else
    cres <- downsampling[2]

  # get the total range of row dendrogram by constructing the total
  rc <- clust$rclust 
  rdend <- dendrolines(rc, hang=rhang, height=-rc$branch.level)
  rdend.range <- range(-rdend$y)
  
  for(k in 1:K) 
    {
    # row dendrogram
    subclust <- rc$subclust[[k]]
    u <- dendrolines(subclust,hang=rhang,
          height=-rc$branch.level[subclust$sup.branch])
    par(mar=c(rgap,2,rgap,2))
    plot(-u$y,u$x,type="l",axes=FALSE,ann=FALSE,
      ylim=c(0.5,nrow.d[k]+0.5+gutter), xlim=rdend.range, yaxs="i" )

    # dataset label
    mtext(dlab[k],side=2,line=1,at=nrow.d[k]/2,cex=.8)
    
    # heatmap
    par(mar=c(rgap,1,rgap,1))
    ik <- cc$unicol[,k]
    ik <- ifelse(ik == 0, NA, ik)

    if( is.null( downsampling[1] ) )
      rres <- nrow(d[[k]]) %/% 300 + 1
    else
      rres <- downsampling[1]
    
    if( is.null(corder) )
      subcols <- cc$order[wa:wb] 
    else
      subcols <- match( corder, cc$labels )

    dx <- t(downsample(
      d[[k]][ subclust$order, ik[ subcols ] ],
      rres, cres, 
      rcenter=FALSE,ccenter=TRUE ))
    
    image( tanh( saturation*(dx)/attr(dx,"sd")),
      zlim=c(-1,1),col=col,
      y=seq(1,nrow(d[[k]]),rres), ylab="", ylim=c(1-0.5,nrow.d[k]+0.5+gutter),
      x=seq(wa,wb,cres), xlab="",
      axes=FALSE)

    ### heatmap axis
    
    # row tick marks
    rtlab <- TRUE
    if( is.numeric(rticks) )
      rti <- seq(rticks[1],nrow.d[k],rticks[1])
    else if( rticks == "l" ) # labels
      {
      rti <- 1:nrow.d[k]; 
      rtlab <- subclust$labels[subclust$order]
      rtlab <- sub("^[0-9][0-9]*:","",rtlab)  # remove prefix
      }
    else
      rti <- pretty(c(1,nrow.d[k],n=5))

    if( !is.character(rtlab) )
      {
      rti <- rti[ rti > 1 & rti < nrow.d[k] ]
      axis(2, at=rti, cex.axis=.8)
      }
    else
      {
      axis(2, at=rti, labels=rtlab,las=1, cex.axis=.8)
      }

    # column tick marks
    if(k == K)
      {
      if(is.null(cticks)) ct <- pretty(c(wa,wb),n=10)
      else ct <- seq(cticks,nu,cticks)
      ct <- ct[ ct > 1 & ct < nu ]
      axis(1, at=ct, cex.axis=.8)
      }
    }

  # column dendrogram
  h <- cc$branch.corr
  hcut <- cres
  for(i in length(h):1)
    {
    if(cc$merge[i,1] > 0 && cc$n.leaf[cc$merge[i,1]] <= hcut)
      h[cc$merge[i,1]] <- h[i]
    if(cc$merge[i,2] > 0 && cc$n.leaf[cc$merge[i,2]] <= hcut )
      h[cc$merge[i,2]] <- h[i]
    }
  u <- dendrolines(cc, hang=chang, height=-h)
  par(mar=c(1,1,2,1))
  if(is.null(corder))
    {
    plot(u$x,-u$y,type="l",axes=FALSE,ann=FALSE,xlim=c(wa-.5,wb+.5),
      xaxs = "i" )
    axis(2,at=c(0,.2,.4,.6,.8,1),labels=c("0",".2",".4",".6",".8","1"),cex.axis=.8)
    }
  else
    {
    plot.new()   # do nothing
    }

  # gene labels
  par(mar=c(0,1,0,1))
  plot.new()
  plot.window(c(wa-.5,wb+.5),c(0,1),xaxs="i")
  if( !missing(clab) )
    {
    if(is.null(corder))
      ordlab <- cc$labels[cc$order]
    else
      ordlab <- cc$labels[match(corder,cc$labels)]

    if(is.character(clab))
      i <- unique(as.vector(na.omit(match( clab, ordlab ))))
    else
      i <- as.integer(clab)

    i <- i[ i >= wa & i <= wb ]

    if(length(i) > 0)
      {
      recordGraphics(
        {
        s <- 0.6*par("cin")[2] / par("pin")[1] * (wb-wa+1) * cex.clab
        j <- labelpack(i, space = s,
          minx = wa - 0.5 + .5*s , maxx = wb + .5 -.5*s  )
        text( x=j, y=rep(0.17, length(i)), cex = cex.clab,
          labels=lab, col=col.lab, adj=c(0,.5), srt=90,xpd=NA )

        # bent ticks
        apply(rbind(i,j),2,
          function(u)lines( c(u[1],u[1],u[2],u[2]), c(0,.05,.1,.15) ))
        },
        list(i=i,nu=nu,cex.clab=cex.clab,lab=ordlab[i],
        col.lab=col.clab,wa=wa,wb=wb),
        getNamespace("graphics")
        )
      }
    }

  # additional matrix
  if( !is.null(rannot) )
    {
    for(k in 1:K)
      {
      if(!is.null(rannot) && !is.null(rannot[[k]]))
        {
        cpos <- unique(sort(c(rannot[[k]])))
        cid <- 1:length(cpos)
        names(cid) <- cpos
        z <- apply( t(rannot[[k]][clust$rclust$subclust[[k]]$order,]), c(1,2), function(u)cid[u] )
        par(mar=c(rgap,1,rgap,1))
        image(z=z, col=cpos,xaxs="i",yaxs="i",axes=F,
          y=seq(1,nrow.d[k]),
          ylim=c(1,nrow.d[k]+gutter))

        }
      }
    if( !is.null(colnames(rannot[[1]])) )
      {
      M <- length(colnames(rannot[[1]]))
      par(mar=c(0,1,0,1))
      plot.new()
      plot.window(c(.5,M+.5),c(0,1),xaxs="i")
      text( x=1:M, y=rep(0,M), labels=colnames(rannot[[1]]),
        cex = cex.clab, adj=c(0,.5), srt=90,xpd=NA)
      }
    }
  # else, we should draw nothing

  invisible(clust)
}

