\name{nclust2}
\alias{nclust2}
\title{Cluster both column and rows}
\description{
Perform both column and row clustering using \code{nclust.mmat}. Multiple
data matrices (linked by the columns) may be used as input.
}
\usage{
nclust2(..., mostvarcol = 0.5, cclust = NULL, labels = NULL,
  center = TRUE, distance = "nzcor.rema",
  link = "ave", branchflip = "nnephew",
  rpwcorr=TRUE, cpwcorr=FALSE )
}
\arguments{
  \item{...}{Data matrices to be clustered; similar format as the
  input to \code{nclust.mmat}.
  }
  \item{mostvarcol}{Proportion most variable columns used as features
  when clustering rows.}
  \item{cclust}{If clustering of columns are already done before,
  e.g. when updating the row clustering, the cluster object (i.e.
  output of \code{nclust.mmat} or the first element of
  \code{nclust2} output list) can be supplied here. Otherwise,
  new cluster will be computed.}
  \code{labels}{Dataset labels to prepend to the row names in each
  dataset (the length has to equal the number of input matrices).
  If omitted,
  1, 2, 3, etc. will be used. The new column names will have one of
  these labels prepended to the original row names, separated by
  a colon \code{':'}. The purpose of this is to avoid conflict
  in row names.}
  \item{center}{Center the columns before clustering rows.}
  \item{distance,link,branchflip}{See \code{nclust.mmat}}
\item{rpwcorr,cpwcorr}{Save row and/or column pairwise correlation matrix.
They can be found as \code{$pwcorr} object inside \code{rclust}
and \code{cclust}. Beware that \code{cpwcorr} can produce extremely
large matrix (the number of columns squared).
}
}
\details{
Column clustering is performed first (if not already supplied
through the \code{cclust} option). The output is used
to select most variable columns. The data is then transposed
and concatenated into a single matrix (with missing values
padding for missing columns in the original input), and
processed by \code{nclust.mmat} to produce a single tree.
The tree is then split into multiple subtrees according
to the original dataset (preserving the ordering, height and relative
topology for each dataset).
}
\value{
A list of two clusters objects. \code{cclust} and
\code{rclust} are the hierarchical clustering dendrograms
for the columns and rows, respectively. Each is an object of
type \code{hclust}, although it contains more information
not found in \code{hclust}.
\item{\code{merge}}{Also in \code{hclust} output. This is an
\code{N-1} by 2 matrix of integers. Each row correspond to
a branch or cluster of the dendrogram. The two numbers are child
nodes of the given branch. If the number is negative,
then its absolute value is the index to the terminal node (the
clustered objects) in the original input order. If the number
if positive, it is an index to another row (another branch)
in the matrix \code{merge}}
\item{height}{The height of the dendrogram. In \code{hclust}, this
is the link score. In \code{nclust}, this can be overwritten by
other branch summaries, and used only as a placeholder.
The link scores are stored somewhere else.}

\code{rclust} contains
an additional object called \code{subclust}, which a list
of subdendograms (in \code{hclust} format, and can be plotted
separately), whose node orderings
and trees can be used to display stratified heatmaps.
}

\author{ Pratyaksha Wirapati }

\seealso{nclust.mmat, coldmap}

\examples{
# see examples in the help for "coldmap"
}
